@@ -7,31 +7,10 @@ import java.math.BigInteger;
 import java.security.GeneralSecurityException;
 import java.security.InvalidAlgorithmParameterException;
 import java.security.PublicKey;
-import java.security.cert.CRL;
-import org.bouncycastle.jce.cert.CertPath;
-import org.bouncycastle.jce.cert.CertPathParameters;
-import org.bouncycastle.jce.cert.CertPathValidatorException;
-import org.bouncycastle.jce.cert.CertPathValidatorResult;
-import org.bouncycastle.jce.cert.CertPathValidatorSpi;
-import org.bouncycastle.jce.cert.CertSelector;
-import org.bouncycastle.jce.cert.CertStore;
-import org.bouncycastle.jce.cert.CertStoreException;
-import java.security.cert.CertificateExpiredException;
-import java.security.cert.CertificateNotYetValidException;
-import org.bouncycastle.jce.cert.PKIXCertPathChecker;
-import org.bouncycastle.jce.cert.PKIXCertPathValidatorResult;
-import org.bouncycastle.jce.cert.PKIXParameters;
-import org.bouncycastle.jce.cert.PolicyQualifierInfo;
-import org.bouncycastle.jce.cert.TrustAnchor;
-import java.security.cert.X509CRL;
-import java.security.cert.X509CRLEntry;
-import org.bouncycastle.jce.cert.X509CRLSelector;
-import org.bouncycastle.jce.cert.X509CertSelector;
-import org.bouncycastle.jce.X509Principal;
-import org.bouncycastle.jce.PrincipalUtil;
-import java.security.cert.X509Certificate;
-import java.security.cert.CRLException;
-import java.security.cert.CertificateEncodingException;
+import java.security.cert.*;
+
+import org.bouncycastle.jce.*;
+
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Date;
@@ -527,7 +506,7 @@ public class PKIXCertPathValidatorSpi extends CertPathValidatorSpi
         }
     }
 
-    private final Set getQualifierSet(ASN1Sequence qualifiers) 
+    private Set getQualifierSet(ASN1Sequence qualifiers) 
         throws CertPathValidatorException
     {
         Set             pq   = new HashSet();
@@ -862,7 +841,7 @@ public class PKIXCertPathValidatorSpi extends CertPathValidatorSpi
                     //
                     cert.verify(workingPublicKey, "BC");
                 }
-                catch (GeneralSecurityException e)
+                catch (Exception e)
                 {
                     throw new CertPathValidatorException("Could not validate certificate signature.", e, certPath, index);
                 }
@@ -2008,7 +1987,7 @@ public class PKIXCertPathValidatorSpi extends CertPathValidatorSpi
      * @return a Collection of all found {@link CRL CRL}
      * objects. May be empty but never <code>null</code>.
      */
-    private final Collection findCRLs(
+    private Collection findCRLs(
         X509CRLSelector crlSelect,
         List            crlStores)
         throws AnnotatedException
