@@ -6,14 +6,11 @@ import org.bouncycastle.asn1.ASN1OctetString;
 import org.bouncycastle.asn1.ASN1Sequence;
 import org.bouncycastle.asn1.ASN1Set;
 import org.bouncycastle.asn1.BERSequence;
-import org.bouncycastle.asn1.DERNull;
-import org.bouncycastle.asn1.DERObject;
 import org.bouncycastle.asn1.DERObjectIdentifier;
 import org.bouncycastle.asn1.DERSet;
 import org.bouncycastle.asn1.cms.ContentInfo;
 import org.bouncycastle.asn1.cms.SignedData;
 import org.bouncycastle.asn1.cms.SignerInfo;
-import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
 import org.bouncycastle.x509.NoSuchStoreException;
 import org.bouncycastle.x509.X509Store;
 
@@ -195,17 +192,19 @@ public class CMSSignedData
 
             for (int i = 0; i != s.size(); i++)
             {
+                SignerInfo info = SignerInfo.getInstance(s.getObjectAt(i));
+                DERObjectIdentifier contentType = signedData.getEncapContentInfo().getContentType();
+
                 if (hashes == null)
                 {
-                    signerInfos.add(new SignerInformation(SignerInfo.getInstance(s.getObjectAt(i)), signedData.getEncapContentInfo().getContentType(), signedContent, null));
+                    signerInfos.add(new SignerInformation(info, contentType, signedContent, null));
                 }
                 else
                 {
-                    SignerInfo info = SignerInfo.getInstance(s.getObjectAt(i));
 
                     byte[] hash = (byte[])hashes.get(info.getDigestAlgorithm().getObjectId().getId());
 
-                    signerInfos.add(new SignerInformation(info, signedData.getEncapContentInfo().getContentType(), null, new BaseDigestCalculator(hash)));
+                    signerInfos.add(new SignerInformation(info, contentType, null, new BaseDigestCalculator(hash)));
                 }
             }
 
@@ -446,20 +445,8 @@ public class CMSSignedData
         Iterator    it = signerInformationStore.getSigners().iterator();
         while (it.hasNext())
         {
-            SignerInformation   signer = (SignerInformation)it.next();
-            AlgorithmIdentifier digAlgId;
-
-            try
-            {
-                digAlgId = makeAlgId(signer.getDigestAlgOID(),
-                                                       signer.getDigestAlgParams());
-            }
-            catch (IOException e)
-            {
-                throw new RuntimeException("encoding error: " + e);
-            }
-
-            digestAlgs.add(digAlgId);
+            SignerInformation signer = (SignerInformation)it.next();
+            digestAlgs.add(CMSSignedHelper.INSTANCE.fixAlgID(signer.getDigestAlgorithmID()));
             vec.add(signer.toSignerInfo());
         }
 
@@ -566,35 +553,4 @@ public class CMSSignedData
         
         return cms;
     }
-
-    private static DERObject makeObj(
-        byte[]  encoding)
-        throws IOException
-    {
-        if (encoding == null)
-        {
-            return null;
-        }
-
-        ASN1InputStream         aIn = new ASN1InputStream(encoding);
-
-        return aIn.readObject();
-    }
-
-    private static AlgorithmIdentifier makeAlgId(
-        String  oid,
-        byte[]  params)
-        throws IOException
-    {
-        if (params != null)
-        {
-            return new AlgorithmIdentifier(
-                            new DERObjectIdentifier(oid), makeObj(params));
-        }
-        else
-        {
-            return new AlgorithmIdentifier(
-                            new DERObjectIdentifier(oid), new DERNull());
-        }
-    }
 }
