@@ -1,7 +1,8 @@
 package org.bouncycastle.cms;
 
+import org.bouncycastle.util.Arrays;
+
 import org.bouncycastle.jce.cert.X509CertSelector;
-import java.util.Arrays;
 
 /**
  * a basic index for a signer.
@@ -11,7 +12,7 @@ public class SignerId
 {
     public int hashCode()
     {
-        int     code = 0;
+        int code = Arrays.hashCode(this.getSubjectKeyIdentifier());
 
         if (this.getSerialNumber() != null)
         {
@@ -23,15 +24,6 @@ public class SignerId
             code ^= this.getIssuerAsString().hashCode();
         }
 
-        byte[] subjectId = this.getSubjectKeyIdentifier();
-        if (subjectId != null)
-        {
-            for (int i = 0; i != subjectId.length; i++)
-            {
-                code ^= ((subjectId[i]) & 0xff) << (i % 4);
-            }
-        }
-        
         return code;
     }
 
@@ -45,23 +37,13 @@ public class SignerId
 
         SignerId id = (SignerId)o;
 
-        return equalsObj(this.getSerialNumber(), id.getSerialNumber())
-            && equalsObj(this.getIssuerAsString(), id.getIssuerAsString())
-            && equalsByteArray(this.getSubjectKeyIdentifier(), id.getSubjectKeyIdentifier());
+        return Arrays.areEqual(this.getSubjectKeyIdentifier(), id.getSubjectKeyIdentifier())
+            && equalsObj(this.getSerialNumber(), id.getSerialNumber())
+            && equalsObj(this.getIssuerAsString(), id.getIssuerAsString());
     }
 
     private boolean equalsObj(Object a, Object b)
     {
         return (a != null) ? a.equals(b) : b == null;
     }
-
-    private boolean equalsByteArray(byte[] a, byte[] b)
-    {
-        if (a != null)
-        {
-            return (b != null) && Arrays.equals(a, b);
-        }
-
-        return (b == null);
-    }
 }
