@@ -7,7 +7,9 @@ import java.util.Enumeration;
 import java.util.Hashtable;
 import java.util.Vector;
 
+import org.bouncycastle.asn1.ASN1Object;
 import org.bouncycastle.asn1.ASN1Sequence;
+import org.bouncycastle.asn1.DERBitString;
 import org.bouncycastle.asn1.DEREncodable;
 import org.bouncycastle.asn1.DERInteger;
 import org.bouncycastle.asn1.DERNull;
@@ -18,6 +20,7 @@ import org.bouncycastle.asn1.cryptopro.CryptoProObjectIdentifiers;
 import org.bouncycastle.asn1.pkcs.PrivateKeyInfo;
 import org.bouncycastle.asn1.sec.ECPrivateKeyStructure;
 import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
+import org.bouncycastle.asn1.x509.SubjectPublicKeyInfo;
 import org.bouncycastle.asn1.x9.X962NamedCurves;
 import org.bouncycastle.asn1.x9.X962Parameters;
 import org.bouncycastle.asn1.x9.X9ECParameters;
@@ -42,8 +45,9 @@ public class JCEECPrivateKey
     private ECParameterSpec ecSpec;
     private boolean         withCompression;
 
-    private Hashtable   pkcs12Attributes = new Hashtable();
-    private Vector      pkcs12Ordering = new Vector();
+    private DERBitString publicKey;
+
+    private PKCS12BagAttributeCarrier attrCarrier = new PKCS12BagAttributeCarrierImpl();
 
     protected JCEECPrivateKey()
     {
@@ -69,6 +73,7 @@ public class JCEECPrivateKey
     public JCEECPrivateKey(
         String                  algorithm,
         ECPrivateKeyParameters  params,
+        JCEECPublicKey          pubKey,
         ECParameterSpec         spec)
     {
         ECDomainParameters      dp = params.getParameters();
@@ -89,6 +94,8 @@ public class JCEECPrivateKey
         {
             this.ecSpec = spec;
         }
+
+        publicKey = getPublicKeyDetails(pubKey);
     }
 
     public JCEECPrivateKey(
@@ -108,8 +115,8 @@ public class JCEECPrivateKey
         this.d = key.d;
         this.ecSpec = key.ecSpec;
         this.withCompression = key.withCompression;
-        this.pkcs12Attributes = key.pkcs12Attributes;
-        this.pkcs12Ordering = key.pkcs12Ordering;
+        this.publicKey = key.publicKey;
+        this.attrCarrier = key.attrCarrier;
     }
 
     JCEECPrivateKey(
@@ -155,6 +162,7 @@ public class JCEECPrivateKey
             ECPrivateKeyStructure   ec = new ECPrivateKeyStructure((ASN1Sequence)info.getPrivateKey());
 
             this.d = ec.getKey();
+            this.publicKey = ec.getPublicKey();
         }
     }
 
@@ -224,15 +232,25 @@ public class JCEECPrivateKey
             params = new X962Parameters(ecP);
         }
 
-        PrivateKeyInfo          info;
-        
-        if (algorithm.equals("ECGOST3410"))
+        PrivateKeyInfo        info;
+        ECPrivateKeyStructure keyStructure;
+
+        if (publicKey != null)
         {
-            info = new PrivateKeyInfo(new AlgorithmIdentifier(CryptoProObjectIdentifiers.gostR3410_2001, params.getDERObject()), new ECPrivateKeyStructure(this.getD()).getDERObject());
+            keyStructure = new ECPrivateKeyStructure(this.getD(), publicKey, params);
         }
         else
         {
-            info = new PrivateKeyInfo(new AlgorithmIdentifier(X9ObjectIdentifiers.id_ecPublicKey, params.getDERObject()), new ECPrivateKeyStructure(this.getD()).getDERObject());
+            keyStructure = new ECPrivateKeyStructure(this.getD(), params);
+        }
+
+        if (algorithm.equals("ECGOST3410"))
+        {
+            info = new PrivateKeyInfo(new AlgorithmIdentifier(CryptoProObjectIdentifiers.gostR3410_2001, params.getDERObject()), keyStructure.getDERObject());
+        }
+        else
+        {
+            info = new PrivateKeyInfo(new AlgorithmIdentifier(X9ObjectIdentifiers.id_ecPublicKey, params.getDERObject()), keyStructure.getDERObject());
         }
         
         try
@@ -323,19 +341,18 @@ public class JCEECPrivateKey
         DERObjectIdentifier oid,
         DEREncodable        attribute)
     {
-        pkcs12Attributes.put(oid, attribute);
-        pkcs12Ordering.addElement(oid);
+        attrCarrier.setBagAttribute(oid, attribute);
     }
 
     public DEREncodable getBagAttribute(
         DERObjectIdentifier oid)
     {
-        return (DEREncodable)pkcs12Attributes.get(oid);
+        return attrCarrier.getBagAttribute(oid);
     }
 
     public Enumeration getBagAttributeKeys()
     {
-        return pkcs12Ordering.elements();
+        return attrCarrier.getBagAttributeKeys();
     }
     
     public void setPointFormat(String style)
@@ -370,4 +387,17 @@ public class JCEECPrivateKey
         return getD().hashCode() ^ engineGetSpec().hashCode();
     }
 
+    private DERBitString getPublicKeyDetails(JCEECPublicKey   pub)
+    {
+        try
+        {
+            SubjectPublicKeyInfo info = SubjectPublicKeyInfo.getInstance(ASN1Object.fromByteArray(pub.getEncoded()));
+
+            return info.getPublicKeyData();
+        }
+        catch (IOException e)
+        {   // should never happen
+            return null;
+        }
+    }
 }
