@@ -12,7 +12,6 @@ import org.bouncycastle.asn1.DEROutputStream;
 import org.bouncycastle.asn1.DERSequence;
 import org.bouncycastle.asn1.cryptopro.GOST3410PublicKeyAlgParameters;
 import org.bouncycastle.asn1.misc.CAST5CBCParameters;
-import org.bouncycastle.asn1.misc.IDEACBCPar;
 import org.bouncycastle.asn1.oiw.ElGamalParameter;
 import org.bouncycastle.asn1.pkcs.DHParameter;
 import org.bouncycastle.asn1.pkcs.PKCS12PBEParams;
@@ -37,6 +36,7 @@ import java.security.AlgorithmParametersSpi;
 import java.security.spec.AlgorithmParameterSpec;
 import java.security.spec.DSAParameterSpec;
 import java.security.spec.InvalidParameterSpecException;
+import java.security.spec.PSSParameterSpec;
 
 public abstract class JDKAlgorithmParameters
     extends AlgorithmParametersSpi
@@ -76,7 +76,7 @@ public abstract class JDKAlgorithmParameters
             String format) 
             throws IOException
         {
-            if (this.isASN1FormatString(format))
+            if (isASN1FormatString(format))
             {
                  return new DEROctetString(engineGetEncoded("RAW")).getEncoded();
             }
@@ -142,7 +142,7 @@ public abstract class JDKAlgorithmParameters
             String format) 
             throws IOException
         {
-            if (this.isASN1FormatString(format))
+            if (isASN1FormatString(format))
             {
                 ASN1InputStream         aIn = new ASN1InputStream(params);
                 
@@ -175,98 +175,6 @@ public abstract class JDKAlgorithmParameters
         }
     }
 
-    public static class IDEAAlgorithmParameters
-        extends JDKAlgorithmParameters
-    {
-        private byte[]  iv;
-
-        protected byte[] engineGetEncoded() 
-            throws IOException
-        {
-            return engineGetEncoded("ASN.1");
-        }
-
-        protected byte[] engineGetEncoded(
-            String format) 
-            throws IOException
-        {
-            if (this.isASN1FormatString(format))
-            {
-                return new IDEACBCPar(engineGetEncoded("RAW")).getEncoded();
-            }
-            
-            if (format.equals("RAW"))
-            {
-                byte[]  tmp = new byte[iv.length];
-
-                System.arraycopy(iv, 0, tmp, 0, iv.length);
-                return tmp;
-            }
-
-            return null;
-        }
-
-        protected AlgorithmParameterSpec localEngineGetParameterSpec(
-            Class paramSpec) 
-            throws InvalidParameterSpecException
-        {
-            if (paramSpec == IvParameterSpec.class)
-            {
-                return new IvParameterSpec(iv);
-            }
-
-            throw new InvalidParameterSpecException("unknown parameter spec passed to IV parameters object.");
-        }
-
-        protected void engineInit(
-            AlgorithmParameterSpec paramSpec) 
-            throws InvalidParameterSpecException
-        {
-            if (!(paramSpec instanceof IvParameterSpec))
-            {
-                throw new InvalidParameterSpecException("IvParameterSpec required to initialise a IV parameters algorithm parameters object");
-            }
-
-            this.iv = ((IvParameterSpec)paramSpec).getIV();
-        }
-
-        protected void engineInit(
-            byte[] params) 
-            throws IOException
-        {
-            this.iv = new byte[params.length];
-
-            System.arraycopy(params, 0, iv, 0, iv.length);
-        }
-
-        protected void engineInit(
-            byte[] params,
-            String format) 
-            throws IOException
-        {
-            if (format.equals("RAW"))
-            {
-                engineInit(params);
-                return;
-            }
-            if (format.equals("ASN.1"))
-            {
-                ASN1InputStream         aIn = new ASN1InputStream(params);
-                IDEACBCPar              oct = new IDEACBCPar((ASN1Sequence)aIn.readObject());
-
-                engineInit(oct.getIV());
-                return;
-            }
-
-            throw new IOException("Unknown parameters format in IV parameters object");
-        }
-
-        protected String engineToString() 
-        {
-            return "IDEA Parameters";
-        }
-    }
-
     public static class RC2AlgorithmParameters
         extends JDKAlgorithmParameters
     {
@@ -323,7 +231,7 @@ public abstract class JDKAlgorithmParameters
             String format) 
             throws IOException
         {
-            if (this.isASN1FormatString(format))
+            if (isASN1FormatString(format))
             {
                 if (parameterVersion == -1)
                 {
@@ -415,7 +323,7 @@ public abstract class JDKAlgorithmParameters
             String format) 
             throws IOException
         {
-            if (this.isASN1FormatString(format))
+            if (isASN1FormatString(format))
             {
                 ASN1InputStream         aIn = new ASN1InputStream(params);
                 RC2CBCParameter         p = RC2CBCParameter.getInstance(aIn.readObject());
@@ -463,7 +371,7 @@ public abstract class JDKAlgorithmParameters
             String format) 
             throws IOException 
         {
-            if (this.isASN1FormatString(format))
+            if (isASN1FormatString(format))
             {
                 return new CAST5CBCParameters(engineGetEncoded(), keyLength).getEncoded();
             }
@@ -517,7 +425,7 @@ public abstract class JDKAlgorithmParameters
             String format) 
             throws IOException
         {
-            if (this.isASN1FormatString(format))
+            if (isASN1FormatString(format))
             {
                 ASN1InputStream         aIn = new ASN1InputStream(params);
                 CAST5CBCParameters      p = CAST5CBCParameters.getInstance(aIn.readObject());
@@ -569,7 +477,7 @@ public abstract class JDKAlgorithmParameters
         protected byte[] engineGetEncoded(
             String format) 
         {
-            if (this.isASN1FormatString(format))
+            if (isASN1FormatString(format))
             {
                 return engineGetEncoded();
             }
@@ -619,7 +527,7 @@ public abstract class JDKAlgorithmParameters
             String format) 
             throws IOException
         {
-            if (this.isASN1FormatString(format))
+            if (isASN1FormatString(format))
             {
                 engineInit(params);
                 return;
@@ -671,7 +579,7 @@ public abstract class JDKAlgorithmParameters
         protected byte[] engineGetEncoded(
             String format) 
         {
-            if (this.isASN1FormatString(format))
+            if (isASN1FormatString(format))
             {
                 return engineGetEncoded();
             }
@@ -737,7 +645,7 @@ public abstract class JDKAlgorithmParameters
             String format) 
             throws IOException
         {
-            if (this.isASN1FormatString(format))
+            if (isASN1FormatString(format))
             {
                 engineInit(params);
             }
@@ -790,7 +698,7 @@ public abstract class JDKAlgorithmParameters
         protected byte[] engineGetEncoded(
             String format) 
         {
-            if (this.isASN1FormatString(format))
+            if (isASN1FormatString(format))
             {
                 return engineGetEncoded();
             }
@@ -849,7 +757,7 @@ public abstract class JDKAlgorithmParameters
             String format) 
             throws IOException
         {
-            if (this.isASN1FormatString(format) || format.equalsIgnoreCase("X.509"))
+            if (isASN1FormatString(format) || format.equalsIgnoreCase("X.509"))
             {
                 engineInit(params);
             }
@@ -902,7 +810,7 @@ public abstract class JDKAlgorithmParameters
         protected byte[] engineGetEncoded(
                 String format)
         {
-            if (this.isASN1FormatString(format) || format.equalsIgnoreCase("X.509"))
+            if (isASN1FormatString(format) || format.equalsIgnoreCase("X.509"))
             {
                 return engineGetEncoded();
             }
@@ -961,7 +869,7 @@ public abstract class JDKAlgorithmParameters
                 String format)
         throws IOException
         {
-            if (this.isASN1FormatString(format) || format.equalsIgnoreCase("X.509"))
+            if (isASN1FormatString(format) || format.equalsIgnoreCase("X.509"))
             {
                 engineInit(params);
             }
@@ -1013,7 +921,7 @@ public abstract class JDKAlgorithmParameters
         protected byte[] engineGetEncoded(
             String format) 
         {
-            if (this.isASN1FormatString(format) || format.equalsIgnoreCase("X.509"))
+            if (isASN1FormatString(format) || format.equalsIgnoreCase("X.509"))
             {
                 return engineGetEncoded();
             }
@@ -1085,7 +993,7 @@ public abstract class JDKAlgorithmParameters
             String format) 
             throws IOException
         {
-            if (this.isASN1FormatString(format) || format.equalsIgnoreCase("X.509"))
+            if (isASN1FormatString(format) || format.equalsIgnoreCase("X.509"))
             {
                 engineInit(params);
             }
@@ -1137,7 +1045,7 @@ public abstract class JDKAlgorithmParameters
         protected byte[] engineGetEncoded(
             String format) 
         {
-            if (this.isASN1FormatString(format) || format.equalsIgnoreCase("X.509"))
+            if (isASN1FormatString(format) || format.equalsIgnoreCase("X.509"))
             {
                 return engineGetEncoded();
             }
@@ -1199,7 +1107,7 @@ public abstract class JDKAlgorithmParameters
             String format) 
             throws IOException
         {
-            if (this.isASN1FormatString(format) || format.equalsIgnoreCase("X.509"))
+            if (isASN1FormatString(format) || format.equalsIgnoreCase("X.509"))
             {
                 engineInit(params);
             }
@@ -1214,4 +1122,112 @@ public abstract class JDKAlgorithmParameters
             return "IES Parameters";
         }
     }
+
+    public static class PSS
+        extends JDKAlgorithmParameters
+    {
+        PSSParameterSpec     currentSpec;
+
+        /**
+         * Return the PKCS#1 ASN.1 structure RSASSA-PSS-params.
+         */
+        protected byte[] engineGetEncoded()
+        {
+            ByteArrayOutputStream   bOut = new ByteArrayOutputStream();
+            DEROutputStream         dOut = new DEROutputStream(bOut);
+            PSSParameterSpec    pssSpec = (PSSParameterSpec)currentSpec;
+            RSASSAPSSparams     pssP = new RSASSAPSSparams(RSASSAPSSparams.DEFAULT_HASH_ALGORITHM, RSASSAPSSparams.DEFAULT_MASK_GEN_FUNCTION, new DERInteger(pssSpec.getSaltLength()), RSASSAPSSparams.DEFAULT_TRAILER_FIELD);
+            try
+            {
+                dOut.writeObject(pssP);
+                dOut.close();
+            }
+            catch (IOException e)
+            {
+                throw new RuntimeException("Error encoding PSSParameters");
+            }
+
+            return bOut.toByteArray();
+        }
+
+        protected byte[] engineGetEncoded(
+            String format)
+        {
+            if (format.equalsIgnoreCase("X.509")
+                    || format.equalsIgnoreCase("ASN.1"))
+            {
+                return engineGetEncoded();
+            }
+
+            return null;
+        }
+
+        protected AlgorithmParameterSpec localEngineGetParameterSpec(
+            Class paramSpec)
+            throws InvalidParameterSpecException
+        {
+            if (paramSpec == PSSParameterSpec.class && currentSpec instanceof PSSParameterSpec)
+            {
+                return currentSpec;
+            }
+
+            throw new InvalidParameterSpecException("unknown parameter spec passed to PSS parameters object.");
+        }
+
+        protected void engineInit(
+            AlgorithmParameterSpec paramSpec)
+            throws InvalidParameterSpecException
+        {
+            if (!(paramSpec instanceof PSSParameterSpec))
+            {
+                throw new InvalidParameterSpecException("PSSParameterSpec required to initialise an PSS algorithm parameters object");
+            }
+
+            this.currentSpec = (PSSParameterSpec)paramSpec;
+        }
+
+        protected void engineInit(
+            byte[] params)
+            throws IOException
+        {
+            ASN1InputStream        aIn = new ASN1InputStream(params);
+
+            try
+            {
+                RSASSAPSSparams pssP = new RSASSAPSSparams((ASN1Sequence)aIn.readObject());
+
+                currentSpec = new PSSParameterSpec(
+                                       pssP.getSaltLength().getValue().intValue());
+            }
+            catch (ClassCastException e)
+            {
+                throw new IOException("Not a valid PSS Parameter encoding.");
+            }
+            catch (ArrayIndexOutOfBoundsException e)
+            {
+                throw new IOException("Not a valid PSS Parameter encoding.");
+            }
+        }
+
+        protected void engineInit(
+            byte[] params,
+            String format)
+            throws IOException
+        {
+            if (format.equalsIgnoreCase("X.509")
+                    || format.equalsIgnoreCase("ASN.1"))
+            {
+                engineInit(params);
+            }
+            else
+            {
+                throw new IOException("Unknown parameter format " + format);
+            }
+        }
+
+        protected String engineToString()
+        {
+            return "PSS Parameters";
+        }
+    }
 }
