@@ -1,13 +1,11 @@
 package org.bouncycastle.cms;
 
 import org.bouncycastle.asn1.ASN1EncodableVector;
-import org.bouncycastle.asn1.BEROctetStringGenerator;
 import org.bouncycastle.asn1.BERSequenceGenerator;
 import org.bouncycastle.asn1.BERSet;
 import org.bouncycastle.asn1.DERInteger;
 import org.bouncycastle.asn1.DERSet;
 import org.bouncycastle.asn1.cms.CMSObjectIdentifiers;
-import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
 import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
 
 import javax.crypto.Cipher;
@@ -118,20 +116,16 @@ public class CMSEnvelopedDataStreamGenerator
         SecretKey           encKey = keyGen.generateKey();
         AlgorithmParameters params = generateParameters(encryptionOID, encKey, encProvider);
 
-        Iterator            it = recipientInfs.iterator();
+        Iterator it = recipientInfoGenerators.iterator();
         ASN1EncodableVector recipientInfos = new ASN1EncodableVector();
         
         while (it.hasNext())
         {
-            RecipientInf            recipient = (RecipientInf)it.next();
+            RecipientInfoGenerator recipient = (RecipientInfoGenerator)it.next();
 
             try
             {
-                recipientInfos.add(recipient.toRecipientInfo(encKey, rand, provider));
-            }
-            catch (IOException e)
-            {
-                throw new CMSException("encoding error.", e);
+                recipientInfos.add(recipient.generate(encKey, rand, provider));
             }
             catch (InvalidKeyException e)
             {
@@ -198,7 +192,7 @@ public class CMSEnvelopedDataStreamGenerator
 
             BERSequenceGenerator eiGen = new BERSequenceGenerator(envGen.getRawOutputStream());
             
-            eiGen.addObject(PKCSObjectIdentifiers.data);
+            eiGen.addObject(CMSObjectIdentifiers.data);
 
             //
             // If params are null we try and second guess on them as some providers don't provide
@@ -212,19 +206,11 @@ public class CMSEnvelopedDataStreamGenerator
             AlgorithmIdentifier encAlgId = getAlgorithmIdentifier(encryptionOID, params);
                         
             eiGen.getRawOutputStream().write(encAlgId.getEncoded());
-            
-            BEROctetStringGenerator octGen = new BEROctetStringGenerator(eiGen.getRawOutputStream(), 0, false);
-            
-            CipherOutputStream      cOut;
-            
-            if (_bufferSize != 0)
-            {
-                cOut = new CipherOutputStream(octGen.getOctetOutputStream(new byte[_bufferSize]), cipher);
-            }
-            else
-            {
-                cOut = new CipherOutputStream(octGen.getOctetOutputStream(), cipher);
-            }
+
+            OutputStream octetStream = CMSUtils.createBEROctetOutputStream(
+                eiGen.getRawOutputStream(), 0, false, _bufferSize);
+
+            CipherOutputStream cOut = new CipherOutputStream(octetStream, cipher);
 
             return new CmsEnvelopedDataOutputStream(cOut, cGen, envGen, eiGen);
         }
