@@ -7,6 +7,7 @@ import org.bouncycastle.crypto.Digest;
 import org.bouncycastle.crypto.engines.RSABlindedEngine;
 import org.bouncycastle.crypto.params.ParametersWithRandom;
 import org.bouncycastle.crypto.signers.PSSSigner;
+import org.bouncycastle.jce.provider.util.NullDigest;
 
 import java.security.AlgorithmParameters;
 import java.security.InvalidKeyException;
@@ -29,9 +30,11 @@ public class JDKPSSSigner
     private PSSParameterSpec       paramSpec;
     private PSSParameterSpec       originalSpec;
     private AsymmetricBlockCipher  signer;
-    private Digest digest;
+    private Digest contentDigest;
+    private Digest mgfDigest;
     private int saltLength;
     private byte trailer;
+    private boolean isRaw;
 
     private PSSSigner pss;
 
@@ -45,27 +48,47 @@ public class JDKPSSSigner
         
         throw new IllegalArgumentException("unknown trailer field");
     }
-    
-    protected JDKPSSSigner(
-        AsymmetricBlockCipher signer,
-        PSSParameterSpec paramSpec)
+
+    private void setupContentDigest()
     {
-        this.signer = signer;
-        
-        if (paramSpec == null)
+        if (isRaw)
         {
-            originalSpec = null;
-            paramSpec = PSSParameterSpec.DEFAULT;
+            this.contentDigest = new NullDigest();
         }
         else
         {
-            originalSpec = paramSpec;
-            this.paramSpec = paramSpec;
+            this.contentDigest = mgfDigest;
         }
+    }
+
+    protected JDKPSSSigner(
+        PSSParameterSpec paramSpecArg)
+    {
+        this(paramSpecArg, false);
+    }
+
+    protected JDKPSSSigner(
+        PSSParameterSpec baseParamSpec,
+        boolean          isRaw)
+    {
+        this.signer = new RSABlindedEngine();
+        this.originalSpec = baseParamSpec;
         
-        this.digest = JCEDigestUtil.getDigest(paramSpec.getDigestAlgorithm());
+        if (baseParamSpec == null)
+        {
+            this.paramSpec = PSSParameterSpec.DEFAULT;
+        }
+        else
+        {
+            this.paramSpec = baseParamSpec;
+        }
+
+        this.mgfDigest = JCEDigestUtil.getDigest(paramSpec.getDigestAlgorithm());
         this.saltLength = paramSpec.getSaltLength();
         this.trailer = getTrailer(paramSpec.getTrailerField());
+        this.isRaw = isRaw;
+
+        setupContentDigest();
     }
     
     protected void engineInitVerify(
@@ -77,7 +100,7 @@ public class JDKPSSSigner
             throw new InvalidKeyException("Supplied key is not a RSAPublicKey instance");
         }
 
-        pss = new PSSSigner(signer, digest, saltLength);
+        pss = new PSSSigner(signer, contentDigest, mgfDigest, saltLength, trailer);
         pss.init(false,
             RSAUtil.generatePublicKeyParameter((RSAPublicKey)publicKey));
     }
@@ -92,7 +115,7 @@ public class JDKPSSSigner
             throw new InvalidKeyException("Supplied key is not a RSAPrivateKey instance");
         }
 
-        pss = new PSSSigner(signer, digest, saltLength, trailer);
+        pss = new PSSSigner(signer, contentDigest, mgfDigest, saltLength, trailer);
         pss.init(true, new ParametersWithRandom(RSAUtil.generatePrivateKeyParameter((RSAPrivateKey)privateKey), random));
     }
 
@@ -105,7 +128,7 @@ public class JDKPSSSigner
             throw new InvalidKeyException("Supplied key is not a RSAPrivateKey instance");
         }
 
-        pss = new PSSSigner(signer, digest, saltLength, trailer);
+        pss = new PSSSigner(signer, contentDigest, mgfDigest, saltLength, trailer);
         pss.init(true, RSAUtil.generatePrivateKeyParameter((RSAPrivateKey)privateKey));
     }
 
@@ -151,41 +174,46 @@ public class JDKPSSSigner
     {
         if (params instanceof PSSParameterSpec)
         {
-            paramSpec = (PSSParameterSpec)params;
+            PSSParameterSpec newParamSpec = (PSSParameterSpec)params;
             
             if (originalSpec != null)
             {
-                if (!JCEDigestUtil.isSameDigest(originalSpec.getDigestAlgorithm(), paramSpec.getDigestAlgorithm()))
+                if (!JCEDigestUtil.isSameDigest(originalSpec.getDigestAlgorithm(), newParamSpec.getDigestAlgorithm()))
                 {
                     throw new InvalidParameterException("parameter must be using " + originalSpec.getDigestAlgorithm());
                 }
             }
-            if (!paramSpec.getMGFAlgorithm().equalsIgnoreCase("MGF1") && !paramSpec.getMGFAlgorithm().equals(PKCSObjectIdentifiers.id_mgf1.getId()))
+            if (!newParamSpec.getMGFAlgorithm().equalsIgnoreCase("MGF1") && !newParamSpec.getMGFAlgorithm().equals(PKCSObjectIdentifiers.id_mgf1.getId()))
             {
                 throw new InvalidParameterException("unknown mask generation function specified");
             }
             
-            if (!(paramSpec.getMGFParameters() instanceof MGF1ParameterSpec))
+            if (!(newParamSpec.getMGFParameters() instanceof MGF1ParameterSpec))
             {
                 throw new InvalidParameterException("unkown MGF parameters");
             }
             
-            MGF1ParameterSpec   mgfParams = (MGF1ParameterSpec)paramSpec.getMGFParameters();
+            MGF1ParameterSpec   mgfParams = (MGF1ParameterSpec)newParamSpec.getMGFParameters();
             
-            if (!JCEDigestUtil.isSameDigest(mgfParams.getDigestAlgorithm(), paramSpec.getDigestAlgorithm()))
+            if (!JCEDigestUtil.isSameDigest(mgfParams.getDigestAlgorithm(), newParamSpec.getDigestAlgorithm()))
             {
                 throw new InvalidParameterException("digest algorithm for MGF should be the same as for PSS parameters.");
             }
             
-            digest = JCEDigestUtil.getDigest(mgfParams.getDigestAlgorithm());
+            Digest newDigest = JCEDigestUtil.getDigest(mgfParams.getDigestAlgorithm());
             
-            if (digest == null)
+            if (newDigest == null)
             {
                 throw new InvalidParameterException("no match on MGF digest algorithm: "+ mgfParams.getDigestAlgorithm());
             }
-            
+
+            this.engineParams = null;
+            this.paramSpec = newParamSpec;
+            this.mgfDigest = newDigest;
             this.saltLength = paramSpec.getSaltLength();
             this.trailer = getTrailer(paramSpec.getTrailerField());
+
+            setupContentDigest();
         }
         else
         {
@@ -230,12 +258,21 @@ public class JDKPSSSigner
         throw new UnsupportedOperationException("engineGetParameter unsupported");
     }
 
+    static public class nonePSS
+        extends JDKPSSSigner
+    {
+        public nonePSS()
+        {
+            super(null, true);
+        }
+    }
+
     static public class PSSwithRSA
         extends JDKPSSSigner
     {
         public PSSwithRSA()
         {
-            super(new RSABlindedEngine(), null);
+            super(null);
         }
     }
     
@@ -244,7 +281,7 @@ public class JDKPSSSigner
     {
         public SHA1withRSA()
         {
-            super(new RSABlindedEngine(), PSSParameterSpec.DEFAULT);
+            super(PSSParameterSpec.DEFAULT);
         }
     }
 
@@ -253,7 +290,7 @@ public class JDKPSSSigner
     {
         public SHA224withRSA()
         {
-            super(new RSABlindedEngine(), new PSSParameterSpec("SHA-224", "MGF1", new MGF1ParameterSpec("SHA-224"), 28, 1));
+            super(new PSSParameterSpec("SHA-224", "MGF1", new MGF1ParameterSpec("SHA-224"), 28, 1));
         }
     }
     
@@ -262,7 +299,7 @@ public class JDKPSSSigner
     {
         public SHA256withRSA()
         {
-            super(new RSABlindedEngine(), new PSSParameterSpec("SHA-256", "MGF1", new MGF1ParameterSpec("SHA-256"), 32, 1));
+            super(new PSSParameterSpec("SHA-256", "MGF1", new MGF1ParameterSpec("SHA-256"), 32, 1));
         }
     }
 
@@ -271,7 +308,7 @@ public class JDKPSSSigner
     {
         public SHA384withRSA()
         {
-            super(new RSABlindedEngine(), new PSSParameterSpec("SHA-384", "MGF1", new MGF1ParameterSpec("SHA-384"), 48, 1));
+            super(new PSSParameterSpec("SHA-384", "MGF1", new MGF1ParameterSpec("SHA-384"), 48, 1));
         }
     }
 
@@ -280,7 +317,7 @@ public class JDKPSSSigner
     {
         public SHA512withRSA()
         {
-            super(new RSABlindedEngine(), new PSSParameterSpec("SHA-512", "MGF1", new MGF1ParameterSpec("SHA-512"), 64, 1));
+            super(new PSSParameterSpec("SHA-512", "MGF1", new MGF1ParameterSpec("SHA-512"), 64, 1));
         }
     }
 }
