@@ -24,13 +24,11 @@ import org.bouncycastle.asn1.x509.X509Extensions;
 import org.bouncycastle.asn1.x509.X509Name;
 import org.bouncycastle.jce.exception.ExtCertPathValidatorException;
 import org.bouncycastle.util.Arrays;
-import org.bouncycastle.util.Selector;
 import org.bouncycastle.x509.ExtendedPKIXBuilderParameters;
 import org.bouncycastle.x509.ExtendedPKIXParameters;
 import org.bouncycastle.x509.X509CRLStoreSelector;
 import org.bouncycastle.x509.X509CertStoreSelector;
 
-import javax.security.auth.x500.X500Principal;
 import java.io.IOException;
 import java.math.BigInteger;
 import java.security.GeneralSecurityException;
@@ -45,8 +43,6 @@ import java.security.cert.PKIXCertPathChecker;
 import java.security.cert.X509CRL;
 import java.security.cert.X509Certificate;
 import java.security.cert.X509Extension;
-import java.security.cert.X509CertSelector;
-import java.security.cert.X509CRLSelector;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Date;
@@ -59,6 +55,8 @@ import java.util.Map;
 import java.util.Set;
 import java.util.Vector;
 
+import javax.security.auth.x500.X500Principal;
+
 public class RFC3280CertPathUtilities
 {
 
@@ -430,7 +428,7 @@ public class RFC3280CertPathUtilities
      * @param certPathCerts      The certificates on the certification path.
      * @return A <code>Set</code> with all keys of possible CRL issuer
      *         certificates.
-     * @throws AnnotatedException if the CRL is no valid or the status cannot be checked or
+     * @throws AnnotatedException if the CRL is not valid or the status cannot be checked or
      *                            some error occurs.
      */
     protected static Set processCRLF(
@@ -594,19 +592,21 @@ public class RFC3280CertPathUtilities
         throws AnnotatedException
     {
         Exception lastException = null;
-        try
+
+        for (Iterator it = deltacrls.iterator(); it.hasNext();)
         {
-            for (Iterator it = deltacrls.iterator(); it.hasNext();)
+            X509CRL crl = (X509CRL)it.next();
+            try
             {
-                X509CRL crl = (X509CRL)it.next();
                 crl.verify(key);
                 return crl;
             }
+            catch (Exception e)
+            {
+                lastException = e;
+            }
         }
-        catch (Exception e)
-        {
-            lastException = e;
-        }
+
         if (lastException != null)
         {
             throw new AnnotatedException("Cannot verify delta CRL.", lastException);
@@ -763,7 +763,6 @@ public class RFC3280CertPathUtilities
 
         if (pkixParams.isUseDeltasEnabled())
         {
-
             // (c) (1)
             if (!deltaCRL.getIssuerX500Principal().equals(completeCRL.getIssuerX500Principal()))
             {
@@ -771,65 +770,75 @@ public class RFC3280CertPathUtilities
             }
 
             // (c) (2)
-            if (completeidp != null)
+            IssuingDistributionPoint deltaidp = null;
+            try
             {
+                deltaidp = IssuingDistributionPoint.getInstance(CertPathValidatorUtilities.getExtensionValue(
+                    deltaCRL, ISSUING_DISTRIBUTION_POINT));
+            }
+            catch (Exception e)
+            {
+                throw new AnnotatedException(
+                    "Issuing distribution point extension from delta CRL could not be decoded.", e);
+            }
 
-                IssuingDistributionPoint deltaidp = null;
-                try
+            boolean match = false;
+            if (completeidp == null)
+            {
+                if (deltaidp == null)
                 {
-                    deltaidp = IssuingDistributionPoint.getInstance(CertPathValidatorUtilities.getExtensionValue(
-                        deltaCRL, ISSUING_DISTRIBUTION_POINT));
+                    match = true;
                 }
-                catch (Exception e)
+            }
+            else
+            {
+                if (completeidp.equals(deltaidp))
                 {
-                    throw new AnnotatedException(
-                        "Issuing distribution point extension from delta CRL could not be decoded.", e);
-                }
-                boolean match = false;
-                if (completeidp == null)
-                {
-                    if (deltaidp == null)
-                    {
-                        match = true;
-                    }
-                }
-                else
-                {
-                    if (completeidp.equals(deltaidp))
-                    {
-                        match = true;
-                    }
-                }
-                if (!match)
-                {
-                    throw new AnnotatedException(
-                        "Issuing distribution point extension from delta CRL and complete CRL does not match.");
+                    match = true;
                 }
             }
+            if (!match)
+            {
+                throw new AnnotatedException(
+                    "Issuing distribution point extension from delta CRL and complete CRL does not match.");
+            }
 
             // (c) (3)
             DERObject completeKeyIdentifier = null;
             try
             {
-                completeKeyIdentifier = CertPathValidatorUtilities
-                    .getExtensionValue(deltaCRL, AUTHORITY_KEY_IDENTIFIER);
+                completeKeyIdentifier = CertPathValidatorUtilities.getExtensionValue(
+                    completeCRL, AUTHORITY_KEY_IDENTIFIER);
             }
             catch (AnnotatedException e)
             {
                 throw new AnnotatedException(
                     "Authority key identifier extension could not be extracted from complete CRL.", e);
             }
+
             DERObject deltaKeyIdentifier = null;
             try
             {
-                deltaKeyIdentifier = CertPathValidatorUtilities.getExtensionValue(deltaCRL, AUTHORITY_KEY_IDENTIFIER);
+                deltaKeyIdentifier = CertPathValidatorUtilities.getExtensionValue(
+                    deltaCRL, AUTHORITY_KEY_IDENTIFIER);
             }
             catch (AnnotatedException e)
             {
                 throw new AnnotatedException(
                     "Authority key identifier extension could not be extracted from delta CRL.", e);
             }
-            if (!(completeKeyIdentifier == null && deltaKeyIdentifier == null) && !completeKeyIdentifier.equals(deltaKeyIdentifier))
+
+            if (completeKeyIdentifier == null)
+            {
+                throw new AnnotatedException("CRL authority key identifier is null.");
+            }
+
+            if (deltaKeyIdentifier == null)
+            {
+                throw new AnnotatedException("Delta CRL authority key identifier is null.");
+            }
+
+            if (!completeKeyIdentifier.equals(deltaKeyIdentifier))
             {
                 throw new AnnotatedException(
                     "Delta CRL authority key identifier does not match complete CRL authority key identifier.");
@@ -1460,6 +1469,7 @@ public class RFC3280CertPathUtilities
         ExtendedPKIXParameters paramsPKIX,
         int index,
         PublicKey workingPublicKey,
+        boolean verificationAlreadyPerformed,
         X500Principal workingIssuerName,
         X509Certificate sign)
         throws ExtCertPathValidatorException
@@ -1469,16 +1479,20 @@ public class RFC3280CertPathUtilities
         //
         // (a) verify
         //
-        try
+        if (!verificationAlreadyPerformed)
         {
-            // (a) (1)
-            //
-            cert.verify(workingPublicKey, "BC");
-        }
-        catch (GeneralSecurityException e)
-        {
-            throw new ExtCertPathValidatorException("Could not validate certificate signature.", e, certPath, index);
-        }
+            try
+            {
+                // (a) (1)
+                //
+                CertPathValidatorUtilities.verifyX509Certificate(cert, workingPublicKey,
+                    paramsPKIX.getSigProvider());
+            }
+            catch (GeneralSecurityException e)
+            {
+                throw new ExtCertPathValidatorException("Could not validate certificate signature.", e, certPath, index);
+            }
+		}
 
         try
         {
@@ -1512,7 +1526,12 @@ public class RFC3280CertPathUtilities
             }
             catch (AnnotatedException e)
             {
-                throw new ExtCertPathValidatorException(e.getMessage(), e.getCause(), certPath, index);
+                Throwable cause = e;
+                if (null != e.getCause())
+                {
+                    cause = e.getCause();
+                }
+                throw new ExtCertPathValidatorException(e.getMessage(), cause, certPath, index);
             }
         }
 
