@@ -31,6 +31,8 @@ import javax.mail.Address;
 import javax.mail.internet.InternetAddress;
 import javax.mail.internet.MimeMessage;
 import javax.mail.internet.MimeMultipart;
+import javax.mail.MessagingException;
+
 import java.io.IOException;
 import java.security.GeneralSecurityException;
 import java.security.PublicKey;
@@ -48,7 +50,6 @@ import java.security.cert.X509Certificate;
 import java.security.interfaces.DSAPublicKey;
 import java.security.interfaces.RSAPublicKey;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.Collection;
 import java.util.Date;
 import java.util.HashMap;
@@ -144,15 +145,12 @@ public class SignedMailValidator
             throws SignedMailValidatorException
     {
         this.certPathReviewerClass = certPathReviewerClass;
-        try
-        {
-            certPathReviewerClass.asSubclass(DEFAULT_CERT_PATH_REVIEWER);
-        }
-        catch (ClassCastException e)
+        boolean isSubclass = DEFAULT_CERT_PATH_REVIEWER.isAssignableFrom(certPathReviewerClass);
+        if(!isSubclass)
         {
             throw new IllegalArgumentException("certPathReviewerClass is not a subclass of " + DEFAULT_CERT_PATH_REVIEWER.getName());
         }
-        
+
         SMIMESigned s;
 
         try
@@ -181,12 +179,24 @@ public class SignedMailValidator
 
             // save "from" addresses from message
             Address[] froms = message.getFrom();
-            fromAddresses = new String[froms.length];
+	    InternetAddress sender = null;
+	    try {
+		    if(message.getHeader("Sender") != null) {
+		     sender = new InternetAddress(message.getHeader("Sender")[0]);
+		    }
+	    }
+	    catch (MessagingException ex) {
+		    //ignore garbage in Sender: header
+	    }
+            fromAddresses = new String[froms.length + (sender!=null?1:0)];
             for (int i = 0; i < froms.length; i++)
             {
                 InternetAddress inetAddr = (InternetAddress) froms[i];
                 fromAddresses[i] = inetAddr.getAddress();
             }
+	    if(sender!=null) {
+		    fromAddresses[froms.length] = sender.getAddress();
+	    }
 
             // initialize results
             results = new HashMap();
@@ -337,17 +347,17 @@ public class SignedMailValidator
                     PKIXCertPathReviewer review;
                     try
                     {
-                        review = (PKIXCertPathReviewer) certPathReviewerClass.newInstance();
+                        review = (PKIXCertPathReviewer)certPathReviewerClass.newInstance();
                     }
                     catch (IllegalAccessException e)
                     {
                         throw new IllegalArgumentException("Cannot instantiate object of type " +
-                                certPathReviewerClass.getName() + ": " + e.getMessage(), e);
+                                certPathReviewerClass.getName() + ": " + e.getMessage());
                     }
                     catch (InstantiationException e)
                     {
                         throw new IllegalArgumentException("Cannot instantiate object of type " +
-                                certPathReviewerClass.getName() + ": " + e.getMessage(), e);
+                                certPathReviewerClass.getName() + ": " + e.getMessage());
                     }
                     review.init(certPath, usedParameters);
                     if (!review.isValidCertPath())
@@ -533,8 +543,8 @@ public class SignedMailValidator
                     ErrorBundle msg = new ErrorBundle(RESOURCE_NAME,
                             "SignedMailValidator.emailFromCertMismatch",
                             new Object[] {
-                                    new UntrustedInput(Arrays
-                                            .toString(fromAddresses)),
+                                    new UntrustedInput(
+                                            addressesToString(fromAddresses)),
                                     new UntrustedInput(certEmails) });
                     errors.add(msg);
                 }
@@ -549,6 +559,28 @@ public class SignedMailValidator
         }
     }
 
+    static String addressesToString(Object[] a)
+    {
+        if (a == null)
+        {
+            return "null";
+        }
+
+        StringBuffer b = new StringBuffer();
+        b.append('[');
+
+        for (int i = 0; i != a.length; i++)
+        {
+            if (i > 0)
+            {
+                b.append(", ");
+            }
+            b.append(String.valueOf(a[i]));
+        }
+
+        return b.append(']').toString();
+    }
+
     public static Date getSignatureTime(SignerInformation signer)
     {
         AttributeTable atab = signer.getSignedAttributes();
@@ -691,7 +723,14 @@ public class SignedMailValidator
             {
                 // add next cert to path
                 X509CertSelector select = new X509CertSelector();
-                select.setSubject(cert.getIssuerX500Principal());
+                try
+                {
+                    select.setSubject(cert.getIssuerX500Principal().getEncoded());
+                }
+                catch (IOException e)
+                {
+                    throw new IllegalStateException(e.toString());
+                }
                 byte[] authKeyIdentBytes = cert.getExtensionValue(X509Extensions.AuthorityKeyIdentifier.getId());
                 if (authKeyIdentBytes != null)
                 {
@@ -738,8 +777,16 @@ public class SignedMailValidator
             else
             {
                 X509CertSelector select = new X509CertSelector();
-                select.setSubject(cert.getIssuerX500Principal());
-                select.setIssuer(cert.getIssuerX500Principal());
+
+                try
+                {
+                    select.setSubject(cert.getIssuerX500Principal().getEncoded());
+                    select.setIssuer(cert.getIssuerX500Principal().getEncoded());
+                }
+                catch (IOException e)
+                {
+                    throw new IllegalStateException(e.toString());
+                }
     
                 boolean userProvided = false;
                 
