@@ -43,7 +43,7 @@ public class PGPPublicKeyRingCollection
      * Build a PGPPublicKeyRingCollection from the passed in input stream.
      *
      * @param in  input stream containing data
-     * @throws IOException if a problem parsinh the base stream occurs
+     * @throws IOException if a problem parsing the base stream occurs
      * @throws PGPException if an object is encountered which isn't a PGPPublicKeyRing
      */
     public PGPPublicKeyRingCollection(
@@ -221,7 +221,7 @@ public class PGPPublicKeyRingCollection
     /**
      * Return the public key ring which contains the key referred to by keyID.
      * 
-     * @param keyID
+     * @param keyID key ID to match against
      * @return the public key ring
      * @throws PGPException
      */
@@ -252,6 +252,18 @@ public class PGPPublicKeyRingCollection
         return null;
     }
     
+    /**
+     * Return true if a key matching the passed in key ID is present, false otherwise.
+     *
+     * @param keyID key ID to look for.
+     * @return true if keyID present, false otherwise.
+     */
+    public boolean contains(long keyID)
+        throws PGPException
+    {
+        return getPublicKey(keyID) != null;
+    }
+
     public byte[] getEncoded() 
         throws IOException
     {
