@@ -10,6 +10,36 @@ public final class Arrays
         // static class, hide constructor
     }
     
+    public static boolean areEqual(
+        boolean[]  a,
+        boolean[]  b)
+    {
+        if (a == b)
+        {
+            return true;
+        }
+
+        if (a == null || b == null)
+        {
+            return false;
+        }
+
+        if (a.length != b.length)
+        {
+            return false;
+        }
+
+        for (int i = 0; i != a.length; i++)
+        {
+            if (a[i] != b[i])
+            {
+                return false;
+            }
+        }
+
+        return true;
+    }
+
     public static boolean areEqual(
         byte[]  a,
         byte[]  b)
@@ -40,6 +70,43 @@ public final class Arrays
         return true;
     }
 
+    /**
+     * A constant time equals comparison - does not terminate early if
+     * test will fail.
+     *
+     * @param a first array
+     * @param b second array
+     * @return true if arrays equal, false otherwise.
+     */
+    public static boolean constantTimeAreEqual(
+        byte[]  a,
+        byte[]  b)
+    {
+        if (a == b)
+        {
+            return true;
+        }
+
+        if (a == null || b == null)
+        {
+            return false;
+        }
+
+        if (a.length != b.length)
+        {
+            return false;
+        }
+
+        int nonEqual = 0;
+
+        for (int i = 0; i != a.length; i++)
+        {
+            nonEqual |= (a[i] ^ b[i]);
+        }
+
+        return nonEqual == 0;
+    }
+
     public static boolean areEqual(
         int[]  a,
         int[]  b)
@@ -102,17 +169,21 @@ public final class Arrays
 
     public static int hashCode(byte[] data)
     {
-        int value = 0;
-
-        if (data != null)
+        if (data == null)
         {
-            for (int i = 0; i != data.length; i++)
-            {
-                value ^= (data[i] & 0xff) << (i % 4);
-            }
+            return 0;
         }
 
-        return value;
+        int i = data.length;
+        int hc = i + 1;
+
+        while (--i >= 0)
+        {
+            hc *= 257;
+            hc ^= data[i];
+        }
+
+        return hc;
     }
 
     public static byte[] clone(byte[] data)
