@@ -1,6 +1,44 @@
 package org.bouncycastle.x509;
 
+import java.io.ByteArrayInputStream;
+import java.io.IOException;
+import java.math.BigInteger;
+import java.net.HttpURLConnection;
+import java.net.InetAddress;
+import java.net.URL;
+import java.security.GeneralSecurityException;
+import java.security.PublicKey;
+import java.security.SignatureException;
+import java.security.cert.CertPath;
+import java.security.cert.CertPathValidatorException;
+import java.security.cert.CertificateExpiredException;
+import java.security.cert.CertificateFactory;
+import java.security.cert.CertificateNotYetValidException;
+import java.security.cert.PKIXCertPathChecker;
+import java.security.cert.PKIXParameters;
+import java.security.cert.PolicyNode;
+import java.security.cert.TrustAnchor;
+import java.security.cert.X509CRL;
+import java.security.cert.X509CRLEntry;
+import java.security.cert.X509CertSelector;
+import java.security.cert.X509Certificate;
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.Date;
+import java.util.Enumeration;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
+import java.util.Set;
+import java.util.Vector;
+
+import javax.security.auth.x500.X500Principal;
+
 import org.bouncycastle.asn1.ASN1InputStream;
+import org.bouncycastle.asn1.ASN1Object;
+import org.bouncycastle.asn1.ASN1OctetString;
 import org.bouncycastle.asn1.ASN1Sequence;
 import org.bouncycastle.asn1.ASN1TaggedObject;
 import org.bouncycastle.asn1.DEREncodable;
@@ -9,6 +47,7 @@ import org.bouncycastle.asn1.DERIA5String;
 import org.bouncycastle.asn1.DERInteger;
 import org.bouncycastle.asn1.DERObject;
 import org.bouncycastle.asn1.DERObjectIdentifier;
+import org.bouncycastle.asn1.DEROctetString;
 import org.bouncycastle.asn1.x509.AccessDescription;
 import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
 import org.bouncycastle.asn1.x509.AuthorityInformationAccess;
@@ -35,46 +74,10 @@ import org.bouncycastle.i18n.filter.UntrustedUrlInput;
 import org.bouncycastle.jce.provider.AnnotatedException;
 import org.bouncycastle.jce.provider.CertPathValidatorUtilities;
 import org.bouncycastle.jce.provider.PKIXNameConstraintValidator;
-import org.bouncycastle.jce.provider.PKIXPolicyNode;
 import org.bouncycastle.jce.provider.PKIXNameConstraintValidatorException;
+import org.bouncycastle.jce.provider.PKIXPolicyNode;
 import org.bouncycastle.x509.extension.X509ExtensionUtil;
 
-import javax.security.auth.x500.X500Principal;
-import java.io.ByteArrayInputStream;
-import java.io.IOException;
-import java.math.BigInteger;
-import java.net.HttpURLConnection;
-import java.net.InetAddress;
-import java.net.URL;
-import java.security.GeneralSecurityException;
-import java.security.PublicKey;
-import java.security.SignatureException;
-import java.security.cert.CertPath;
-import java.security.cert.CertPathValidatorException;
-import java.security.cert.CertificateExpiredException;
-import java.security.cert.CertificateFactory;
-import java.security.cert.CertificateNotYetValidException;
-import java.security.cert.PKIXCertPathChecker;
-import java.security.cert.PKIXParameters;
-import java.security.cert.PolicyNode;
-import java.security.cert.TrustAnchor;
-import java.security.cert.X509CRL;
-import java.security.cert.X509CRLEntry;
-import java.security.cert.X509CRLSelector;
-import java.security.cert.X509CertSelector;
-import java.security.cert.X509Certificate;
-import java.util.ArrayList;
-import java.util.Collection;
-import java.util.Date;
-import java.util.Enumeration;
-import java.util.HashMap;
-import java.util.HashSet;
-import java.util.Iterator;
-import java.util.List;
-import java.util.Map;
-import java.util.Set;
-import java.util.Vector;
-
 /**
  * PKIXCertPathReviewer<br>
  * Validation of X.509 Certificate Paths. Tries to find as much errors in the Path as possible.
@@ -741,7 +744,8 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
                 }
                 try
                 {
-                    cert.verify(trustPublicKey);
+                    CertPathValidatorUtilities.verifyX509Certificate(cert, trustPublicKey,
+                        pkixParams.getSigProvider());
                 }
                 catch (SignatureException e)
                 {
@@ -757,6 +761,11 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
         catch (CertPathReviewerException cpre)
         {
             addError(cpre.getErrorMessage());
+        } catch (Throwable t) {
+            ErrorBundle msg = new ErrorBundle(RESOURCE_NAME,
+                    "CertPathReviewer.unknown",
+                    new Object[] {new UntrustedInput(t.getMessage()), new UntrustedInput(t)});
+            addError(msg);
         }
         
         if (trust != null)
@@ -857,7 +866,8 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
             {
                 try
                 {
-                    cert.verify(workingPublicKey, "BC");
+                    CertPathValidatorUtilities.verifyX509Certificate(cert, workingPublicKey,
+                        pkixParams.getSigProvider());
                 }
                 catch (GeneralSecurityException ex)
                 {
@@ -870,7 +880,8 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
             {
                 try
                 {
-                    cert.verify(cert.getPublicKey(), "BC");
+                    CertPathValidatorUtilities.verifyX509Certificate(cert, cert.getPublicKey(),
+                        pkixParams.getSigProvider());
                     ErrorBundle msg = new ErrorBundle(RESOURCE_NAME,"CertPathReviewer.rootKeyIsValidButNotATrustAnchor");
                     addError(msg, index);
                 }
@@ -2059,7 +2070,7 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
                 int numbOfCrls = nonMatchingCrlNames.size();
                 ErrorBundle msg = new ErrorBundle(RESOURCE_NAME,
                         "CertPathReviewer.noCrlInCertstore",
-                        new Object[] {new UntrustedInput(crlselect.getIssuers()),
+                        new Object[] {new UntrustedInput(crlselect.getIssuerNames()),
                                       new UntrustedInput(nonMatchingCrlNames),
                                       new Integer(numbOfCrls)});
                 addNotification(msg,index);
@@ -2452,7 +2463,7 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
             {
                 HttpURLConnection conn = (HttpURLConnection) url.openConnection();
                 conn.setUseCaches(false);
-                conn.setConnectTimeout(2000);
+                //conn.setConnectTimeout(2000);
                 conn.setDoInput(true);
                 conn.connect();
                 if (conn.getResponseCode() == HttpURLConnection.HTTP_OK)
@@ -2477,7 +2488,7 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
         return result;
     }
     
-    private Collection getTrustAnchors(X509Certificate cert, Set trustanchors) throws CertPathReviewerException
+    protected Collection getTrustAnchors(X509Certificate cert, Set trustanchors) throws CertPathReviewerException
     {
         Collection trustColl = new ArrayList();
         Iterator it = trustanchors.iterator();
@@ -2487,6 +2498,20 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
         try
         {
             certSelectX509.setSubject(getEncodedIssuerPrincipal(cert).getEncoded());
+            byte[] ext = cert.getExtensionValue(X509Extensions.AuthorityKeyIdentifier.getId());
+
+            if (ext != null)
+            {
+                ASN1OctetString oct = (ASN1OctetString)ASN1Object.fromByteArray(ext);
+                AuthorityKeyIdentifier authID = AuthorityKeyIdentifier.getInstance(ASN1Object.fromByteArray(oct.getOctets()));
+
+                certSelectX509.setSerialNumber(authID.getAuthorityCertSerialNumber());
+                byte[] keyID = authID.getKeyIdentifier();
+                if (keyID != null)
+                {
+                    certSelectX509.setSubjectKeyIdentifier(new DEROctetString(keyID).getEncoded());
+                }
+            }
         }
         catch (IOException ex)
         {
