@@ -33,6 +33,8 @@ public class RegressionTest
         new RijndaelTest(),
         new SerpentTest(),
         new CamelliaTest(),
+        new CamelliaLightTest(),
+        new DigestRandomNumberTest(),
         new SkipjackTest(),
         new BlowfishTest(),
         new TwofishTest(),
@@ -89,13 +91,18 @@ public class RegressionTest
         new Salsa20Test(),
         new CMacTest(),
         new EAXTest(),
+        new GCMTest(),
         new HCFamilyTest(),
+        new HCFamilyVecTest(),
         new ISAACTest(),
         new NoekeonTest(),
         new VMPCKSA3Test(),
         new VMPCMacTest(),
         new VMPCTest(),
+        new Grainv1Test(),
+        new Grain128Test(),
         //new NaccacheSternTest(),
+        new SRP6Test(),
         new NullTest()
     };
 
