@@ -3,7 +3,6 @@ package org.bouncycastle.mail.smime.test;
 import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.FileNotFoundException;
-import java.io.FileOutputStream;
 import java.io.FilterOutputStream;
 import java.io.IOException;
 import java.io.OutputStream;
@@ -282,6 +281,30 @@ public class SMIMESignedTest
         multipartMixedTest(part1, part2);
     }
 
+    public void testSHA1WithRSAPSS()
+        throws Exception
+    {
+        rsaPSSTest(SMIMESignedGenerator.DIGEST_SHA1);
+    }
+
+    public void testSHA224WithRSAPSS()
+        throws Exception
+    {
+        rsaPSSTest(SMIMESignedGenerator.DIGEST_SHA224);
+    }
+
+    public void testSHA256WithRSAPSS()
+        throws Exception
+    {
+        rsaPSSTest(SMIMESignedGenerator.DIGEST_SHA256);
+    }
+
+    public void testSHA384WithRSAPSS()
+        throws Exception
+    {
+        rsaPSSTest(SMIMESignedGenerator.DIGEST_SHA384);
+    }
+
     public void multipartMixedTest(MimeBodyPart part1, MimeBodyPart part2)
         throws Exception
     {
@@ -385,6 +408,39 @@ public class SMIMESignedTest
         verifySigners(newS.getCertificatesAndCRLs("Collection", "BC"), newS.getSignerInfos());
     }
 
+    public void testMD5WithRSAAddSignersSHA1()
+        throws Exception
+    {
+        MimeMultipart smm = generateMultiPartRsa(SMIMESignedGenerator.DIGEST_SHA1, msg);
+        SMIMESigned   s = new SMIMESigned(smm);
+
+        List certList = new ArrayList();
+
+        certList.add(_signCert);
+        certList.add(_origCert);
+
+        CertStore certs = CertStore.getInstance("Collection",
+                        new CollectionCertStoreParameters(certList), "BC");
+
+        SMIMESignedGenerator gen = new SMIMESignedGenerator();
+
+        gen.addSigner(_signKP.getPrivate(), _signCert, SMIMESignedGenerator.DIGEST_MD5);
+        
+        gen.addSigners(s.getSignerInfos());
+
+        gen.addCertificatesAndCRLs(certs);
+
+        smm = gen.generate(msg, "BC");
+
+        SMIMESigned newS =  new SMIMESigned(gen.generate(msg, "BC"));
+
+        verifyMessageBytes(msg, newS.getContent());
+
+        verifySigners(newS.getCertificatesAndCRLs("Collection", "BC"), newS.getSignerInfos());
+
+        assertEquals("\"md5,sha1\"", getMicAlg(smm));
+    }
+
     public void testSHA1WithRSACanonicalization()
         throws Exception
     {
@@ -713,6 +769,20 @@ public class SMIMESignedTest
         assertTrue(attrCerts.getMatches(null).contains(attrCert));
     }
 
+    private void rsaPSSTest(String digestOID)
+        throws Exception
+    {
+        MimeMultipart     smm = generateMultiPartRsaPSS(digestOID, msg, null);
+        SMIMESignedParser s = new SMIMESignedParser(smm);
+        CertStore         certs = s.getCertificatesAndCRLs("Collection", "BC");
+
+        assertEquals(getDigestOid(s.getSignerInfos()), digestOID);
+
+        verifyMessageBytes(msg, s.getContent());
+
+        verifySigners(certs, s.getSignerInfos());
+    }
+
     private MimeBodyPart generateBinaryPart() throws MessagingException
     {
         byte[] content = new byte[] { 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 10, 11, 12, 13, 14, 10, 10, 15, 16 };   
@@ -751,6 +821,35 @@ public class SMIMESignedTest
         return gen.generate(msg, "BC");
     }
 
+    private MimeMultipart generateMultiPartRsaPSS(
+        String digestOid,
+        MimeBodyPart msg,
+        Date         signingTime)
+        throws Exception
+    {
+        List certList = new ArrayList();
+
+        certList.add(_signCert);
+        certList.add(_origCert);
+
+        CertStore certs = CertStore.getInstance("Collection",
+                        new CollectionCertStoreParameters(certList), "BC");
+
+        ASN1EncodableVector signedAttrs = generateSignedAttributes();
+
+        if (signingTime != null)
+        {
+            signedAttrs.add(new Attribute(CMSAttributes.signingTime, new DERSet(new Time(signingTime))));
+        }
+
+        SMIMESignedGenerator gen = new SMIMESignedGenerator();
+
+        gen.addSigner(_signKP.getPrivate(), _signCert, SMIMESignedGenerator.ENCRYPTION_RSA_PSS, digestOid, new AttributeTable(signedAttrs), null);
+        gen.addCertificatesAndCRLs(certs);
+
+        return gen.generate(msg, "BC");
+    }
+
     private MimeMultipart generateMultiPartGost(
         MimeBodyPart msg)
         throws Exception
@@ -1032,6 +1131,16 @@ public class SMIMESignedTest
         verifySigners(s.getCertificatesAndCRLs("Collection", "BC"), s.getSignerInfos());
     }
 
+    public void testExtraNlInPostamble()
+        throws Exception
+    {
+        MimeMessage message = loadMessage("extra-nl.eml");
+
+        SMIMESigned s = new SMIMESigned((MimeMultipart)message.getContent());
+
+        verifySigners(s.getCertificatesAndCRLs("Collection", "BC"), s.getSignerInfos());
+    }
+
     public void testSignAttachmentOnly()
         throws Exception
     {
